# SPDX-License-Identifier: Apache-2.0

import json

import jinja2

from osism import utils as osism_utils
from osism.tasks import netbox, openstack
from osism.tasks.conductor.netbox import (
    get_device_oob_ip,
    get_nb_device_query_list_ironic,
)
from osism.tasks.conductor.utils import (
    deep_compare,
    deep_decrypt,
    deep_merge,
    get_vault,
)


driver_params = {
    "ipmi": {
        "address": "ipmi_address",
        "port": "ipmi_port",
        "password": "ipmi_password",
        "username": "ipmi_username",
    },
    "redfish": {
        "address": "redfish_address",
        "password": "redfish_password",
        "username": "redfish_username",
    },
}


def _prepare_node_attributes(device, get_ironic_parameters):
    # Get base node attributes (no decryption needed)
    node_attributes = get_ironic_parameters()

    # Create vault instance for Custom Field decryption
    vault = get_vault()

    # Decrypt and merge ironic_parameters Custom Field if present
    if (
        "ironic_parameters" in device.custom_fields
        and device.custom_fields["ironic_parameters"]
    ):
        ironic_parameters_cf = device.custom_fields["ironic_parameters"]
        deep_decrypt(ironic_parameters_cf, vault)
        deep_merge(node_attributes, ironic_parameters_cf)

    # Decrypt secrets Custom Field
    node_secrets = device.custom_fields.get("secrets", {})
    if node_secrets is None:
        node_secrets = {}
    deep_decrypt(node_secrets, vault)

    if (
        "driver" in node_attributes
        and node_attributes["driver"] in driver_params.keys()
    ):
        if "driver_info" in node_attributes:
            unused_drivers = [
                driver
                for driver in driver_params.keys()
                if driver != node_attributes["driver"]
            ]
            for key in list(node_attributes["driver_info"].keys()):
                for driver in unused_drivers:
                    if key.startswith(driver + "_"):
                        node_attributes["driver_info"].pop(key, None)

            username_key = driver_params[node_attributes["driver"]]["username"]
            if username_key in node_attributes["driver_info"]:
                node_attributes["driver_info"][username_key] = (
                    jinja2.Environment(loader=jinja2.BaseLoader())
                    .from_string(node_attributes["driver_info"][username_key])
                    .render(
                        remote_board_username=str(
                            node_secrets.get("remote_board_username", "admin")
                        )
                    )
                )

            password_key = driver_params[node_attributes["driver"]]["password"]
            if password_key in node_attributes["driver_info"]:
                node_attributes["driver_info"][password_key] = (
                    jinja2.Environment(loader=jinja2.BaseLoader())
                    .from_string(node_attributes["driver_info"][password_key])
                    .render(
                        remote_board_password=str(
                            node_secrets.get("remote_board_password", "password")
                        )
                    )
                )

            address_key = driver_params[node_attributes["driver"]]["address"]
            if address_key in node_attributes["driver_info"]:
                oob_ip_result = get_device_oob_ip(device)
                if oob_ip_result:
                    oob_ip, _ = oob_ip_result
                    node_attributes["driver_info"][address_key] = (
                        jinja2.Environment(loader=jinja2.BaseLoader())
                        .from_string(node_attributes["driver_info"][address_key])
                        .render(remote_board_address=oob_ip)
                    )
    node_attributes.update({"resource_class": device.name})
    if "extra" not in node_attributes:
        node_attributes["extra"] = {}
    if "instance_info" in node_attributes and node_attributes["instance_info"]:
        node_attributes["extra"].update(
            {"instance_info": json.dumps(node_attributes["instance_info"])}
        )
    if (
        "netplan_parameters" in device.custom_fields
        and device.custom_fields["netplan_parameters"]
    ):
        node_attributes["extra"].update(
            {
                "netplan_parameters": json.dumps(
                    device.custom_fields["netplan_parameters"]
                )
            }
        )
    if (
        "frr_parameters" in device.custom_fields
        and device.custom_fields["frr_parameters"]
    ):
        node_attributes["extra"].update(
            {"frr_parameters": json.dumps(device.custom_fields["frr_parameters"])}
        )

    return node_attributes


def sync_ironic(request_id, get_ironic_parameters, node_name=None, force_update=False):
    if node_name:
        osism_utils.push_task_output(
            request_id,
            f"Starting NetBox device synchronisation with ironic for node {node_name}\n",
        )
    else:
        osism_utils.push_task_output(
            request_id,
            "Starting NetBox device synchronisation with ironic\n",
        )
    devices = set()
    nb_device_query_list = get_nb_device_query_list_ironic()
    for nb_device_query in nb_device_query_list:
        devices |= set(netbox.get_devices(**nb_device_query))

    # Filter devices by node_name if specified
    if node_name:
        devices = {dev for dev in devices if dev.name == node_name}
        if not devices:
            osism_utils.push_task_output(
                request_id,
                f"Node {node_name} not found in NetBox\n",
            )
            osism_utils.finish_task_output(request_id, rc=1)
            return

    # NOTE: Find nodes in Ironic which are no longer present in NetBox and remove them
    device_names = {dev.name for dev in devices}
    nodes = openstack.baremetal_node_list()

    # Filter nodes by node_name if specified
    if node_name:
        nodes = [node for node in nodes if node["name"] == node_name]

    for node in nodes:
        osism_utils.push_task_output(
            request_id, f"Looking for {node['name']} in NetBox\n"
        )
        if node["name"] not in device_names:
            if (
                not node["instance_uuid"]
                and node["provision_state"]
                in ["enroll", "manageable", "available", "clean failed"]
                and node["power_state"] in ["power off", None]
            ):
                osism_utils.push_task_output(
                    request_id,
                    f"Cleaning up baremetal node not found in NetBox: {node['name']}\n",
                )
                if node["provision_state"] == "clean failed":
                    # NOTE: Move node to manageable to allow deletion
                    node = openstack.baremetal_node_set_provision_state(
                        node["uuid"], "manage"
                    )
                    node = openstack.baremetal_node_wait_for_nodes_provision_state(
                        node["uuid"], "manageable"
                    )
                for port in openstack.baremetal_port_list(
                    details=False, attributes=dict(node_uuid=node["uuid"])
                ):
                    openstack.baremetal_port_delete(port.id)
                openstack.baremetal_node_delete(node["uuid"])
            else:
                osism_utils.push_task_output(
                    f"Cannot remove baremetal node because it is still provisioned or running: {node}"
                )

    # NOTE: Find nodes in NetBox which are not present in Ironic and add them
    for device in devices:
        osism_utils.push_task_output(
            request_id, f"Looking for {device.name} in ironic\n"
        )

        node_interfaces = list(netbox.get_interfaces_by_device(device.name))

        node_attributes = _prepare_node_attributes(device, get_ironic_parameters)
        ports_attributes = [
            dict(address=interface.mac_address)
            for interface in node_interfaces
            if interface.enabled and not interface.mgmt_only and interface.mac_address
        ]

        lock = osism_utils.create_redlock(
            key=f"lock_osism_tasks_conductor_sync_ironic-{device.name}",
            auto_release_time=600,
        )
        if lock.acquire(timeout=120):
            try:
                osism_utils.push_task_output(
                    request_id, f"Processing device {device.name}\n"
                )
                node = openstack.baremetal_node_show(device.name, ignore_missing=True)
                if not node:
                    osism_utils.push_task_output(
                        request_id, f"Creating baremetal node for {device.name}\n"
                    )
                    node = openstack.baremetal_node_create(device.name, node_attributes)
                else:
                    # NOTE: Check whether the baremetal node needs to be updated
                    node_updates = {}
                    deep_compare(node_attributes, node, node_updates)
                    if "driver_info" in node_updates:
                        # NOTE: The password is not returned by ironic, so we cannot make a comparision and it would always be updated. Therefore we pop it from the dictionary
                        password_key = driver_params[node_attributes["driver"]][
                            "password"
                        ]
                        if password_key in node_updates["driver_info"]:
                            node_updates["driver_info"].pop(password_key, None)
                            if not node_updates["driver_info"]:
                                node_updates.pop("driver_info", None)
                    if node_updates or force_update:
                        osism_utils.push_task_output(
                            request_id,
                            f"Updating baremetal node for {device.name} with {node_updates}\n",
                        )
                        # NOTE: Do the actual updates with all values in node_attributes. Otherwise nested dicts like e.g. driver_info will be overwritten as a whole and contain only changed values
                        node = openstack.baremetal_node_update(
                            node["uuid"], node_attributes
                        )

                node_ports = openstack.baremetal_port_list(
                    details=False, attributes=dict(node_uuid=node["uuid"])
                )
                # NOTE: Baremetal ports are only required for (i)pxe boot
                for port_attributes in ports_attributes:
                    port_attributes.update({"node_id": node["uuid"]})
                    port = [
                        port
                        for port in node_ports
                        if port_attributes["address"].upper() == port["address"].upper()
                    ]
                    if not port:
                        osism_utils.push_task_output(
                            request_id,
                            f"Creating baremetal port with MAC address {port_attributes['address']} for {device.name}\n",
                        )
                        openstack.baremetal_port_create(port_attributes)
                    else:
                        node_ports.remove(port[0])
                for node_port in node_ports:
                    # NOTE: Delete remaining ports not found in NetBox
                    osism_utils.push_task_output(
                        request_id,
                        f"Deleting baremetal port with MAC address {node_port['address']} for {device.name}\n",
                    )
                    openstack.baremetal_port_delete(node_port["id"])

                node_validation = openstack.baremetal_node_validate(node["uuid"])
                if node_validation["management"].result:
                    osism_utils.push_task_output(
                        request_id,
                        f"Validation of management interface successful for baremetal node for {device.name}\n",
                    )
                    if node["provision_state"] in ["enroll", "clean failed"]:
                        osism_utils.push_task_output(
                            request_id,
                            f"Transitioning baremetal node to manageable state for {device.name}\n",
                        )
                        node = openstack.baremetal_node_set_provision_state(
                            node["uuid"], "manage"
                        )
                        node = openstack.baremetal_node_wait_for_nodes_provision_state(
                            node["uuid"], "manageable"
                        )
                        osism_utils.push_task_output(
                            request_id,
                            f"Baremetal node for {device.name} is manageable\n",
                        )
                    if node_validation["boot"].result:
                        osism_utils.push_task_output(
                            request_id,
                            f"Validation of boot interface successful for baremetal node for {device.name}\n",
                        )
                        if node["provision_state"] == "manageable":
                            osism_utils.push_task_output(
                                request_id,
                                f"Transitioning baremetal node to available state for {device.name}\n",
                            )
                            node = openstack.baremetal_node_set_provision_state(
                                node["uuid"], "provide"
                            )
                            node = (
                                openstack.baremetal_node_wait_for_nodes_provision_state(
                                    node["uuid"], "available"
                                )
                            )
                            osism_utils.push_task_output(
                                request_id,
                                f"Baremetal node for {device.name} is available\n",
                            )
                    else:
                        osism_utils.push_task_output(
                            request_id,
                            f"Validation of boot interface failed for baremetal node for {device.name}\nReason: {node_validation['boot'].reason}\n",
                        )
                        if node["provision_state"] == "available":
                            # NOTE: Demote node to manageable
                            osism_utils.push_task_output(
                                request_id,
                                f"Transitioning baremetal node to manageable state for {device.name}\n",
                            )
                            node = openstack.baremetal_node_set_provision_state(
                                node["uuid"], "manage"
                            )
                            node = (
                                openstack.baremetal_node_wait_for_nodes_provision_state(
                                    node["uuid"], "manageable"
                                )
                            )
                            osism_utils.push_task_output(
                                request_id,
                                f"Baremetal node for {device.name} is manageable\n",
                            )
                else:
                    osism_utils.push_task_output(
                        request_id,
                        f"Validation of management interface failed for baremetal node for {device.name}\nReason: {node_validation['management'].reason}\n",
                    )
            except Exception as exc:
                osism_utils.push_task_output(
                    request_id,
                    f"Could not fully synchronize device {device.name} with ironic: {exc}\n",
                )
            finally:
                lock.release()

        else:
            osism_utils.push_task_output(
                "Could not acquire lock for node {device.name}"
            )

    osism_utils.finish_task_output(request_id, rc=0)


def sync_netbox_from_ironic(request_id, node_name=None, netbox_filter=None):
    """Sync Ironic node states to NetBox (including secondaries)

    This function synchronizes the state of Ironic baremetal nodes to NetBox.
    It updates three custom fields in NetBox:
    - provision_state: The current provision state of the node
    - power_state: The current power state of the node
    - maintenance: Whether the node is in maintenance mode

    The sync is performed for the primary NetBox instance and all configured
    secondary NetBox instances. NetBox instances can be filtered by URL substring.

    Args:
        request_id: The Celery task request ID for output tracking
        node_name: Optional name of a specific node to sync. If None, all nodes are synced.
        netbox_filter: Optional URL filter (substring match). If provided, only NetBox
                      instances whose base_url contains this substring will be updated.
                      Example: 'primary' matches 'https://primary-netbox.example.com'
    """
    filter_msg = f" (NetBox filter: {netbox_filter})" if netbox_filter else ""
    if node_name:
        osism_utils.push_task_output(
            request_id,
            f"Starting Ironic to NetBox synchronisation for node {node_name}{filter_msg}\n",
        )
    else:
        osism_utils.push_task_output(
            request_id,
            f"Starting Ironic to NetBox synchronisation{filter_msg}\n",
        )

    # Get all Ironic nodes
    nodes = openstack.baremetal_node_list()

    # Filter by node_name if specified
    if node_name:
        nodes = [n for n in nodes if n["name"] == node_name]
        if not nodes:
            osism_utils.push_task_output(
                request_id,
                f"Node {node_name} not found in Ironic\n",
            )
            osism_utils.finish_task_output(request_id, rc=1)
            return

    # Sync each node to NetBox (including secondaries)
    for node in nodes:
        osism_utils.push_task_output(
            request_id,
            f"Syncing state of {node['name']} to NetBox (including secondaries)\n",
        )

        # Update all three states (each function handles primary + secondary NetBox instances)
        # Pass netbox_filter to only update matching NetBox instances
        netbox.set_provision_state(
            node["name"], node["provision_state"], netbox_filter=netbox_filter
        )
        netbox.set_power_state(
            node["name"], node["power_state"], netbox_filter=netbox_filter
        )
        netbox.set_maintenance(
            node["name"], state=node["is_maintenance"], netbox_filter=netbox_filter
        )

    osism_utils.finish_task_output(request_id, rc=0)



# SPDX-License-Identifier: Apache-2.0

import os
import time
from collections.abc import Callable
from typing import Any

from kombu import Connection, Exchange, Queue
from kombu.mixins import ConsumerMixin
from loguru import logger
import json
import requests

from osism.tasks import netbox
from osism import settings

# Multiple exchanges for different OpenStack services
EXCHANGES_CONFIG = {
    "ironic": {
        "exchange": "ironic",
        "routing_key": "ironic_versioned_notifications.info",
        "queue": "osism-listener-ironic",
    },
    "nova": {
        "exchange": "nova",
        "routing_key": "nova_versioned_notifications.info",
        "queue": "osism-listener-nova",
    },
    "neutron": {
        "exchange": "neutron",
        "routing_key": "neutron_versioned_notifications.info",
        "queue": "osism-listener-neutron",
    },
    "cinder": {
        "exchange": "cinder",
        "routing_key": "cinder_versioned_notifications.info",
        "queue": "osism-listener-cinder",
    },
    "keystone": {
        "exchange": "keystone",
        "routing_key": "keystone_versioned_notifications.info",
        "queue": "osism-listener-keystone",
    },
    "glance": {
        "exchange": "glance",
        "routing_key": "glance_versioned_notifications.info",
        "queue": "osism-listener-glance",
    },
}

# Legacy constants for backward compatibility
EXCHANGE_NAME = "ironic"
ROUTING_KEY = "ironic_versioned_notifications.info"
QUEUE_NAME = "osism-listener-ironic"
BROKER_URI = os.getenv("BROKER_URI")


class BaremetalEvents:
    # References:
    #
    # * https://docs.openstack.org/ironic/latest/admin/notifications.html#available-notifications
    # * https://docs.openstack.org/ironic/latest/user/states.html
    # * https://docs.openstack.org/ironic/latest/_images/states.svg

    def __init__(self) -> None:
        self._handler: dict[
            str, dict[str, dict[str, dict[str, Callable[[dict[Any, Any]], None]]]]
        ] = {
            "baremetal": {
                "node": {
                    "power_set": {"end": self.node_power_set_end},
                    "power_state_corrected": {
                        "success": self.node_power_state_corrected_success
                    },
                    "maintenance_set": {"end": self.node_maintenance_set_end},
                    "provision_set": {
                        "start": self.node_provision_set_start,
                        "end": self.node_provision_set_end,
                        "success": self.node_provision_set_success,
                    },
                    "delete": {"end": self.node_delete_end},
                    "create": {"end": self.node_create_end},
                },
            }
        }

    def get_object_data(self, payload: dict[Any, Any]) -> Any:
        return payload["ironic_object.data"]

    def get_handler(self, event_type: str) -> Callable[[dict[Any, Any]], None]:
        event_type_keys = event_type.split(".")
        try:
            handler = self._handler[event_type_keys[0]][event_type_keys[1]][
                event_type_keys[2]
            ][event_type_keys[3]]
        except KeyError:

            def default_handler(payload: dict[Any, Any]) -> None:
                object_data = self.get_object_data(payload)
                name = object_data["name"]
                logger.info(event_type + f" ## {name}")

            handler = default_handler
        return handler

    def node_power_set_end(self, payload: dict[Any, Any]) -> None:
        object_data = self.get_object_data(payload)
        name = object_data["name"]
        logger.info(
            f"baremetal.node.power_set.end ## {name} ## {object_data['power_state']}"
        )
        netbox.set_power_state.delay(name, object_data["power_state"])

    def node_power_state_corrected_success(self, payload: dict[Any, Any]) -> None:
        object_data = self.get_object_data(payload)
        name = object_data["name"]
        logger.info(
            f"baremetal.node.power_state_corrected.success ## {name} ## {object_data['power_state']}"
        )
        netbox.set_power_state.delay(name, object_data["power_state"])

    def node_maintenance_set_end(self, payload: dict[Any, Any]) -> None:
        object_data = self.get_object_data(payload)
        name = object_data["name"]
        logger.info(
            f"baremetal.node.maintenance_set.end ## {name} ## {object_data['maintenance']}"
        )
        netbox.set_maintenance.delay(name, state=object_data["maintenance"])

    def node_provision_set_success(self, payload: dict[Any, Any]) -> None:
        # A provision status was successfully set, update it in the NetBox
        object_data = self.get_object_data(payload)
        name = object_data["name"]
        logger.info(
            f"baremetal.node.provision_set.success ## {name} ## {object_data['provision_state']}"
        )
        netbox.set_provision_state.delay(name, object_data["provision_state"])

    def node_provision_set_start(self, payload: dict[Any, Any]) -> None:
        object_data = self.get_object_data(payload)
        name = object_data["name"]
        logger.info(
            f"baremetal.node.provision_set.start ## {name} ## {object_data['provision_state']}"
        )
        netbox.set_provision_state.delay(name, object_data["provision_state"])

    def node_provision_set_end(self, payload: dict[Any, Any]) -> None:
        object_data = self.get_object_data(payload)
        name = object_data["name"]
        logger.info(
            f"baremetal.node.provision_set.end ## {name} ## {object_data['provision_state']}"
        )
        netbox.set_provision_state.delay(name, object_data["provision_state"])

    def node_delete_end(self, payload: dict[Any, Any]) -> None:
        object_data = self.get_object_data(payload)
        name = object_data["name"]
        logger.info(f"baremetal.node.delete.end ## {name}")
        netbox.set_provision_state.delay(name, None)
        netbox.set_power_state.delay(name, None)

    def node_create_end(self, payload: dict[Any, Any]) -> None:
        object_data = self.get_object_data(payload)
        name = object_data["name"]
        logger.info(f"baremetal.node.create.end ## {name}")
        netbox.set_provision_state.delay(name, object_data["provision_state"])
        netbox.set_power_state.delay(name, object_data["power_state"])


class NotificationsDump(ConsumerMixin):
    def __init__(self, connection):
        self.connection = connection
        self.baremetal_events = BaremetalEvents()
        self.osism_api_session: None | requests.Session = None
        self.osism_baremetal_api_url: None | str = None
        self.websocket_manager = None

        if settings.OSISM_API_URL:
            logger.info("Setting up OSISM API")
            self.osism_api_session = requests.Session()
            self.osism_baremetal_api_url = (
                settings.OSISM_API_URL.rstrip("/") + "/notifications/baremetal"
            )

        # Import event_bridge for WebSocket forwarding
        try:
            from osism.services.event_bridge import event_bridge

            self.event_bridge = event_bridge
            logger.info("Event bridge connected to RabbitMQ listener")
        except ImportError:
            logger.warning("Event bridge not available")
            self.event_bridge = None

        return

    def get_consumers(self, consumer, channel):
        consumers = []

        # Create consumers for all configured exchanges
        for service_name, config in EXCHANGES_CONFIG.items():
            try:
                exchange = Exchange(config["exchange"], type="topic", durable=False)
                queue = Queue(
                    config["queue"],
                    exchange,
                    routing_key=config["routing_key"],
                    durable=False,
                    auto_delete=True,
                    no_ack=True,
                )
                consumers.append(consumer(queue, callbacks=[self.on_message]))
                logger.info(
                    f"Configured consumer for {service_name} exchange: {config['exchange']}"
                )
            except Exception as e:
                logger.error(f"Failed to configure consumer for {service_name}: {e}")

        if not consumers:
            logger.warning(
                "No consumers configured, falling back to legacy ironic consumer"
            )
            # Fallback to legacy configuration
            exchange = Exchange(EXCHANGE_NAME, type="topic", durable=False)
            queue = Queue(
                QUEUE_NAME,
                exchange,
                routing_key=ROUTING_KEY,
                durable=False,
                auto_delete=True,
                no_ack=True,
            )
            consumers.append(consumer(queue, callbacks=[self.on_message]))

        return consumers

    def on_message(self, body, message):
        data = json.loads(body["oslo.message"])

        # Log event with service type detection
        event_type = data.get("event_type", "")
        service_type = event_type.split(".")[0] if event_type else "unknown"

        # Enhanced logging for different event types
        payload_info = {}
        if "payload" in data:
            payload = data["payload"]

            # Extract relevant info based on service type
            if service_type == "baremetal" and "ironic_object.data" in payload:
                ironic_data = payload["ironic_object.data"]
                payload_info = {
                    k: v
                    for k, v in ironic_data.items()
                    if k in ["name", "provision_state", "power_state"]
                }
            elif service_type in ["compute", "nova"] and "nova_object.data" in payload:
                nova_data = payload["nova_object.data"]
                payload_info = {
                    k: v
                    for k, v in nova_data.items()
                    if k in ["uuid", "host", "state", "task_state"]
                }
            elif service_type in ["network", "neutron"]:
                # Neutron events might have different structures
                payload_info = {"service": "neutron"}
            else:
                # Generic payload info
                payload_info = {"service": service_type}

        logger.debug(f"{event_type}: {payload_info}")
        logger.info(f"Received {service_type} event: {event_type}")

        # Send event to WebSocket clients via event bridge
        if self.event_bridge:
            try:
                logger.debug(f"Forwarding event to WebSocket via bridge: {event_type}")
                self.event_bridge.add_event(data["event_type"], data["payload"])
                logger.debug(f"Successfully forwarded event to bridge: {event_type}")
            except Exception as e:
                logger.error(f"Error forwarding event to bridge: {e}")
                logger.error(
                    f"Event data was: {data['event_type']} - {data.get('payload', {}).get('ironic_object.data', {}).get('name', 'unknown')}"
                )

        if self.osism_api_session:
            tries = 1
            max_tries = 3
            while tries <= max_tries:
                logger.info(
                    f"Trying to deliver notification to {self.osism_baremetal_api_url} (Try: {tries}/{max_tries})\n"
                )
                try:
                    response = self.osism_api_session.post(
                        self.osism_baremetal_api_url,
                        timeout=5,
                        json=dict(
                            priority=data["priority"],
                            event_type=data["event_type"],
                            timestamp=data["timestamp"],
                            publisher_id=data["publisher_id"],
                            message_id=data["message_id"],
                            payload=data["payload"],
                        ),
                    )
                    if response.status_code == 204:
                        logger.info(
                            f"Successfully delivered notification to {self.osism_baremetal_api_url} (Try: {tries}/{max_tries})"
                        )
                        break
                    else:
                        response.raise_for_status()
                except requests.ConnectionError:
                    logger.error(f"Error connecting to {self.osism_baremetal_api_url}")
                except requests.Timeout:
                    logger.error(
                        f"Timeout reached while connecting to {self.osism_baremetal_api_url}"
                    )
                except requests.HTTPError as e:
                    logger.error(
                        f"Received HTTP status code {e.response.status_code} while connecting to {self.osism_baremetal_api_url}"
                    )
                    if e.response.status_code <= 500:
                        logger.error(
                            f"Received HTTP status code {e.response.status_code} indicates a client side error, giving up early"
                        )
                        break

                logger.error(
                    f"Failed to deliver notification to {self.osism_baremetal_api_url} ({tries}/{max_tries})"
                )
                tries += 1
                if tries > max_tries:
                    logger.error(
                        f"Giving up delivering notification to {self.osism_baremetal_api_url} with data:\n"
                        + json.dumps(data)
                    )
                else:
                    time.sleep(pow(3, tries - 1))

        else:
            handler = self.baremetal_events.get_handler(data["event_type"])
            handler(data["payload"])


def main():
    while True:
        try:
            with Connection(BROKER_URI, connect_timeout=30.0) as connection:
                connection.connect()
                NotificationsDump(connection).run()
        except ConnectionRefusedError:
            logger.error("Connection with broker refused. Retry in 60 seconds.")
            time.sleep(60)


if __name__ == "__main__":
    main()
